<?php

/**
 * @file
 * Semantic Similarity module Create, Retrieve, Update and delete functions.
 */

/**
 * Returns sorted list of published node ids for which content has not been
 * exported to the filesystem yet. If $nid specified, it will be excluded from
 * the list.
 *
 * @param int $nid
 *   The node id to exclude from the list. Default to NULL.
 *
 * @return array $nids
 *   The num-keyed array of node ids not in the corpus.
 */
function _semantic_similarity_get_nids_not_exported($nid = NULL) {
  $nids = array();

  // Get all published nids from database but the one passed in parameter
  $sql = ($nid)
    ? "SELECT nid from {node} WHERE status = 1 AND nid <> %d ORDER BY nid"
    : "SELECT nid from {node} WHERE status = 1 ORDER BY nid";
  $result = db_query($sql, $nid);
  while ($row = db_fetch_array($result)) {
    $nids[] = $row['nid'];
  }

  // Parse nids from corpus/filesystem and remove their match from $nids
  $corpus = file_scan_directory(SEMANTIC_SIMILARITY_CORPUS_PATH, '([0-9]+)', array('.', '..', 'CVS'), 0, FALSE, 'name');
  ksort($corpus);

  // Filter out corpus nids from $nids
  foreach ($corpus as $corpus_k => $corpus_v) {
    if ($nids_k = array_search($corpus_k, $nids)) {
      unset($nids[$nids_k]);
    }
  }

  return $nids;
}

/**
 * Returns ids of published nodes which are missing some semantic similarity
 * scores with other nodes.
 *
 * @return array
 *   A num-keyed array of node ids missing some semantic scores.
 */
function _semantic_similarity_get_nids_missing_scores() {
  $nids = array();

  // Get number of possible scores per nid
  $result = db_query("SELECT COUNT(n.nid) AS nb_nodes FROM {node} AS n WHERE status = 1");
  $row = db_fetch_array($result);
  $max_nb_scores_per_node = $row['nb_nodes'];

  // Get nids for which number of scores is below number of possible scores per nid
  $sql = "
    SELECT n.nid AS nid, COUNT(n.nid) AS count
    FROM {node} AS n
    LEFT OUTER JOIN {node_semantic_similarity} AS nss ON (n.nid = nss.snid OR n.nid = nss.tnid)
    WHERE n.status = 1
    GROUP BY n.nid
    HAVING count < %d
  ";
  $result = db_query($sql, $max_nb_scores_per_node);
  while ($row = db_fetch_array($result)) {
    $nids[] = $row['nid'];
  }

  return $nids;
}

/**
 * Returns ids of published nodes which were changed (created or updated) since
 * last cron.
 *
 * @return array
 *   A num-keyed array of node ids changed since last cron.
 */
function _semantic_similarity_get_nids_changed_since_last_cron() {
  $nids = array();

  // Get all nids created/updated since last cron
  $sql = "
    SELECT n.nid AS nid FROM {node} AS n WHERE n.status = 1 AND n.changed > %d
  ";
  $result = db_query($sql, variable_get('cron_last', 0));
  while ($row = db_fetch_array($result)) {
    $nids[] = $row['nid'];
  }

  return $nids;
}

/**
 * Saves nodes' semantic similarity scores into database.
 *
 * @param array &$scores
 *   The node ids nids-keyed array of scores.
 */
function _semantic_similarity_save_scores(&$scores) {
  // Delete previous scores from database
  _semantic_similarity_delete_scores();

  // Insert new scores into database
  $sql = "INSERT INTO {node_semantic_similarity} (snid, tnid, score) VALUES ";
  foreach ($scores as $snid => $tnids) {
    foreach ($tnids as $tnid => $score) {
      $values[] = sprintf("(%d, %d, %f)", $snid, $tnid, $score);
    }
  }
  $result = db_query($sql . implode(', ', $values));
}

/**
 * Deletes node's / all semantic similarity scores from database.
 *
 * @param int $nid
 *   The node id for which we delete scores. If NULL we delete all scores.
 */
function _semantic_similarity_delete_scores($nid = NULL) {
  if ($nid) {
    // Delete previous scores from database
    $sql = "DELETE FROM {node_semantic_similarity} WHERE snid = %d OR tnid = %d";
    $result = db_query($sql, $nid, $nid);
  }
  else {
    $sql = "TRUNCATE {node_semantic_similarity}";
    $result = db_query($sql);
  }
}

/**
 * Returns pair of most similar node ids (source and target node) and their
 * score.
 *
 * @param string $order
 *   The order of the result set. Either DESC for most or ASC for least
 *   similars.
 * @param int $limit
 *   The number of records to return.
 *
 * @return array $data
 *   A num-keyed 2D array of nids with their title and score. Format is
 *   array(array('snid', 'tnid', 'score', 'title1', 'title2')).
 */
function _semantic_similarity_get_scored_nids($order = 'DESC', $limit) {
  $data = array();

  $sql = "
    SELECT nss.snid, nss.tnid, nss.score, n1.title AS title1, n2.title AS title2
    FROM {node_semantic_similarity} nss
    LEFT JOIN {node} n1 ON n1.nid = nss.snid
    LEFT JOIN {node} n2 ON n2.nid = nss.tnid
    WHERE n1.status = 1 AND n2.status = 1
    ORDER BY nss.score %s
    ";
  $result = db_query_range(db_rewrite_sql($sql, 'nss', 'nid'), array($order), 0, $limit);
  while ($row = db_fetch_array($result)) {
    $data[] = array(
      'snid'   => $row['snid'],
      'tnid'   => $row['tnid'],
      'score'  => $row['score'],
      'title1' => $row['title1'],
      'title2' => $row['title2'],
    );
  }

  return $data;
}

/**
 * Returns node id most/least similar node ids and their score.
 *
 * @param int $nid
 *   The node id for which we want the semantically similar node ids.
 * @param string $order
 *   The order of the result set. Either DESC for most or ASC for least
 *   similars.
 * @param int $limit
 *   The number of records to return.
 *
 * @return array $data
 *   A num-keyed 2D array of nids with their title and score. Format is
 *   array(array('nid', 'score', 'title')).
 */
function _semantic_similarity_get_node_scored_nids($nid, $order = 'DESC', $limit) {
  $data = array();

  $sql = "
    SELECT n.nid, nss.score, n.title
    FROM {node_semantic_similarity} nss
    JOIN {node} n ON (n.nid = nss.snid OR n.nid = nss.tnid)
    WHERE n.status = 1
      AND n.nid <> %d
      AND (nss.tnid = %d OR nss.snid = %d)
    ORDER BY nss.score %s
  ";
  $result = db_query_range(db_rewrite_sql($sql), array($nid, $nid, $nid, $order), 0, $limit);
  while ($row = db_fetch_array($result)) {
    $data[] = array(
      'nid' => $row['nid'],
      'score' => $row['score'],
      'title' => $row['title']
    );
  }

  return $data;
}
